const express = require('express');

const xml = require('js2xmlparser');

const cityApp = express();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const conn = require(`${libs}/db/monet`);

function response(req, res) {
    if (req.query.format === 'csv') {
        res.csv(req.result.data);
    } else if (req.query.format === 'xml') {
        res.send(xml('result', JSON.stringify({ city: req.result.data })));
    } else {
        res.json({ result: req.result.data });
    }
}

cityApp.get('/', (req, res) => {
    conn.query('SELECT * FROM municipios', true).then(
        (dbResult) => {
            log.debug(dbResult);
            req.result = dbResult;
            response(req, res);
        },
        (dbError) => {
            log.error(`SQL query execution error: ${dbError.message}`);
            // FIXME: change response to HTTP 501 status
            res.json({ error: 'An internal error has occurred' }).end();
        }
    );
});

cityApp.get('/:id', (req, res) => {
    const citySql = 'SELECT * FROM municipios WHERE pk_municipio_id = ?';
    const cityId = parseInt(req.params.id, 10);
    conn.prepare(citySql, true).then((dbQuery) => {
        dbQuery.exec([cityId]).then(
            (dbResult) => {
                log.debug(dbResult);
                req.result = dbResult;
                response(req, res);
            },
            (dbError) => {
                log.error(`SQL query execution error: ${dbError.message}`);
                // FIXME: change response to HTTP 501 status
                res.json({ error: 'An internal error has occurred' }).end();
            }
        );
    });
});

cityApp.get('/ibge/:id', (req, res) => {
    const citySql = 'SELECT * FROM municipios WHERE codigo_ibge = ?';
    const cityIbgeCode = req.params.id;
    conn.prepare(citySql, true).then((dbQuery) => {
        dbQuery.exec([cityIbgeCode]).then(
            (dbResult) => {
                log.debug(dbResult);
                req.result = dbResult;
                response(req, res);
            },
            (dbError) => {
                log.error(`SQL query execution error: ${dbError.message}`);
                // FIXME: change response to HTTP 501 status
                res.json({ error: 'An internal error has occurred' }).end();
            }
        );
    });
});

cityApp.get('/state/:id', (req, res) => {
    const citySql = 'SELECT * FROM municipios WHERE fk_estado_id = ?';
    const stateId = parseInt(req.params.id, 10);
    conn.prepare(citySql, true).then((dbQuery) => {
        dbQuery.exec([stateId]).then(
            (dbResult) => {
                log.debug(dbResult);
                req.result = dbResult;
                response(req, res);
            },
            (dbError) => {
                log.error(`SQL query execution error: ${dbError.message}`);
                // FIXME: change response to HTTP 501 status
                res.json({ error: 'An internal error has occurred' }).end();
            }
        );
    });
});

module.exports = cityApp;
