const express = require('express');

const xml = require('js2xmlparser');

const stateApp = express();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const conn = require(`${libs}/db/monet`);

function response(req, res) {
    if (req.query.format === 'csv') {
        res.csv(req.result.data);
    } else if (req.query.format === 'xml') {
        res.send(xml('result', JSON.stringify({ state: req.result.data })));
    } else {
        res.json({ result: req.result.data });
    }
}

stateApp.get('/', (req, res, next) => {
    const stateSql = 'SELECT * FROM estados';
    conn.query(stateSql, true).then(
        (dbResult) => {
            log.debug(dbResult);
            req.result = dbResult;
            response(req, res);
        },
        (dbError) => {
            log.error(`SQL query execution error: ${dbError.message}`);
            // FIXME: change response to HTTP 501 status
            res.json({ error: 'An internal error has occurred' }).end();
        }
    );
});

stateApp.get('/:id', (req, res, next) => {
    const stateSql = 'SELECT * FROM estados WHERE pk_estado_id = ?';
    const stateId = parseInt(req.params.id, 10);
    conn.prepare(stateSql, true).then((dbQuery) => {
        dbQuery.exec([stateId]).then(
            (dbResult) => {
                log.debug(dbResult);
                req.result = dbResult;
                response(req, res);
            },
            (dbError) => {
                log.error(`SQL query execution error: ${dbError.message}`);
                // FIXME: change response to HTTP 501 status
                res.json({ error: 'An internal error has occurred' }).end();
            }
        );
    });
});

stateApp.get('/region/:id', (req, res, next) => {
    const stateSql = 'SELECT * FROM estados WHERE fk_regiao_id = ?';
    const regionId = parseInt(req.params.id, 10);
    conn.prepare(stateSql, true).then((dbQuery) => {
        dbQuery.exec([regionId]).then(
            (dbResult) => {
                log.debug(dbResult);
                req.result = dbResult;
                response(req, res);
            },
            (dbError) => {
                log.error(`SQL query execution error: ${dbError.message}`);
                // FIXME: change response to HTTP 501 status
                res.json({ error: 'An internal error has occurred' }).end();
            }
        );
    });
});

module.exports = stateApp;
